/** @file scdata_a.h
 * Functions and structures to analyse data from gathered
 * from SCslave's.
 */

#ifndef SCDATA_A_H_
#define SCDATA_A_H_

#include "sccommon.h"
#include "scdata.h"

#include "wlcstdc.h"

/**
 * Minimum time that has to pass between SCslave data records
 * from one SCslave. If there is a sequence of data records that
 * don't have smaller time gaps then only the first one counts.
 * Default valuse is 3 s.
 */
extern int SC_min_change_time_sec;


/**
 * The value that SCs_data_analis can return is 0..SC_score_numb
 */
extern int SC_score_numb;

/**
 * Initializes the analyse engine. This function does nothing
 * for now and always returns 0.
 * @return 0 if successful
 */
int SCs_analis_init();

/**
 * Analyses a data record from SCslave and returns its score.
 * Remember to pass SCslave data records in the same order
 * as they ware gathered by the SCrunner program.
 * @param data data record from SCslave
 * @return analyse score which can be 0..SC_score_numb
 */
int SCs_data_analis(const struct SCs_data* data);

/**
 * Resets the analyse engine. I has to be done if you want to
 * start an analyse another sequence of SCslave data records
 */
void SCs_analis_reset();

#endif /* SCDATA_A_H_ */
