#include "scdb.h"



MYSQL* conn = NULL;

int query_size = 100;

struct SCdb_cfg* make_SCdb_cfg(const char* ini_path)
{
	struct SCdb_cfg *cfg = malloc( sizeof(struct SCdb_cfg) );

	cfg->d = iniparser_load(ini_path);
	if(!cfg->d)
		return NULL;
	
	cfg->host = iniparser_getstr(cfg->d, "main:host");
	if(!cfg->host)
		return NULL;
	cfg->user = iniparser_getstr(cfg->d, "main:user");
	if(!cfg->user)
		return NULL;
	cfg->passwd = iniparser_getstr(cfg->d, "main:passwd");
	if(!cfg->passwd)
		return NULL;
	cfg->db_name = iniparser_getstr(cfg->d, "main:db_name");
	if(!cfg->db_name)
		return NULL;

	return cfg;
}

int SCdb_init(const struct SCdb_cfg *cfg)
{
	conn = mysql_init(NULL);
	if(conn  == NULL)
	{
		fprintf(stderr, "MySQL:%s\n", mysql_error(conn));
		return -1;
	}
	if(mysql_real_connect(conn, cfg->host, cfg->user,
		cfg->passwd, cfg->db_name, 0,
		NULL, CLIENT_MULTI_STATEMENTS | CLIENT_INTERACTIVE) == NULL)
	{
		fprintf(stderr, "MySQL:%s\n", mysql_error(conn));
		return -1;
	}
	return 0;
}

void SCdb_free()
{
	mysql_close(conn);
}

int SCdb_reset_connection(const struct SCdb_cfg *cfg)
{
	mysql_close(conn);
	conn = mysql_init(NULL);
	if(conn  == NULL)
	{
		fprintf(stderr, "MySQL:%s\n", mysql_error(conn));
		return -1;
	}
	if(mysql_real_connect(conn, cfg->host, cfg->user,
		cfg->passwd, cfg->db_name, 0,
		NULL, CLIENT_MULTI_STATEMENTS | CLIENT_INTERACTIVE) == NULL)
	{
		fprintf(stderr, "MySQL:%s\n", mysql_error(conn));
		return -1;
	}
	return 0;
}
SCDB_RES* SCdb_store_result() { return mysql_store_result(conn); }

int SCdb_query(const char* query)
{
	if(mysql_query(conn, query) != 0)
	{
		fprintf(stderr, "MySQL:%s\n", mysql_error(conn));
		return -1;
	}

	return 0;
}

int SCdb_insert_wl(const char* date)
{
	char query[query_size];
	sprintf(query, "call insert_wl('%s')", date);
	return SCdb_query(query);
}

int SCdb_insert_SCs_data( int over, const char* date,
	const struct SCs_data* data )
{
	char query[query_size];
	char strb[TIMELEN];
	if(over)
		sprintf( query,
			"call insert_SCdata_override('%s', '%s', %u, %u)",
			date, print_tms(strb, &(data->time)),
			data->code, data->input );
	else
		sprintf(query,
			"call insert_SCdata('%s', '%s', %u, %u)",
			date, print_tms(strb, &(data->time)),
			data->code, data->input);
	
	return SCdb_query(query);
}

int SCdb_get_wls( int year, int month )
{
	char query[query_size];
	sprintf(query, "call show_wl(%u, %u)", year, month);
	return SCdb_query(query);
}

int SCdb_get_planed_shift( int year, int month )
{
	char query[query_size];
	sprintf(query, "call show_planed_shift(%u, %u)", year, month);
	return SCdb_query(query);
}

int SCdb_get_workers()
{
	char query[query_size];
	strcpy(query, "call show_workers()");
	return SCdb_query(query);
}

int SCdb_insert_worker( const char* first_name, const char* last_name )
{
	char query[query_size];
	sprintf( query,
		"call insert_worker('%s', '%s')",
		first_name, last_name );
	return SCdb_query(query);
}

int SCdb_delete_worker( const char* first_name, const char* last_name )
{
	char query[query_size];
	sprintf( query,
		"call delete_worker('%s', '%s')",
		first_name, last_name);
	return SCdb_query(query);
}

 int SCdb_delete_alldata( const char* date )
 {
	char query[query_size];
	sprintf(query, "call delete_alldata('%s')", date);
	return SCdb_query(query);
}

int SCdb_delete_shift( const char* date )
{
	char query[query_size];
	sprintf(query, "call delete_shift('%s')", date);
	return SCdb_query(query);
}

int SCdb_insert_shift( const char* date,
	const char* first_name, const char* last_name,
	int station, int shift )
{
	char query[query_size];
	sprintf( query,
		"call insert_shift ('%s', '%s', '%s', %u, %u)",
		date, first_name, last_name, station, shift);
	return SCdb_query(query);
}

int SCdb_get_shift(const char* date)
{
	char query[query_size];
	sprintf( query, "call show_shift('%s')", date );
	return SCdb_query(query);
}

int SCdb_get_work( const char* first_name, const char* last_name,
	const char* date )
{
	char query[query_size];
	sprintf( query,
		"call show_work('%s', '%s', '%s')",
		first_name, last_name, date );
	return SCdb_query(query);
}

int SCdb_get_input( const char* date,
	const char* tfrom, const char* tto, int ws )
{
	char query[query_size];
	sprintf( query,
		"call show_day_input('%s', '%s', '%s', %u)",
		date, tfrom, tto, ws );
	return SCdb_query(query);
}



