#include "schard.h"

int SCs_fd[SCS_NUMB_MAX] = { -1 };
char	*SCs_check_cmd = "c",
	*SCs_data_cmd = "d";

struct termios SCs_tio;

int SC_hard_init()
{
	bzero( &SCs_tio, sizeof(struct termios) );
	SCs_tio.c_cflag = B4800 | CS8 | PARENB | CREAD | CLOCAL;
	SCs_tio.c_iflag = 0;
	SCs_tio.c_lflag = 0;
	SCs_tio.c_cc[VTIME] = 10;
	SCs_tio.c_cc[VMIN] = 0;
	return 0;
}

int SCs_init( const char* path, int mach_code )
{
	if(mach_code > SCS_NUMB_MAX || mach_code < 1)
		return -1;
	SCs_fd[mach_code-1] = open( path, O_RDWR | O_NOCTTY );
	if( SCs_fd[mach_code-1] < 0 )
		return -1;
	tcflush(SCs_fd[mach_code-1], TCIFLUSH);
	tcsetattr(SCs_fd[mach_code-1], TCSANOW, &SCs_tio);

	if( SCs_data_cycle(mach_code) == -2 )
		return -1;

	return 0;
}	

int SCs_data_cycle(int mach_code)
{
	unsigned char buf[1] = { 127 };
	
	if(mach_code > SCS_NUMB_MAX || mach_code < 1)
		return not_init;
	if( SCs_fd[mach_code-1] == -1 )
		return not_init;

	tcflush(SCs_fd[mach_code-1], TCIFLUSH);
	
	if( write( SCs_fd[mach_code-1], SCs_check_cmd, 1 ) < 0 )
		return com_error;
	if( read( SCs_fd[mach_code-1], buf, 1 ) <= 0)
		return com_error;
	if( buf[0] == 0x00 )
		return no_new_data;
	else if( buf[0] != 0xFF )
		return com_error;

	if( write( SCs_fd[mach_code-1], SCs_data_cmd, 1 ) < 0 )
		return com_error;
	buf[0] = 0;
	buf[0] = 0xFF;
	if( read( SCs_fd[mach_code-1], buf, 3 ) < 0 )
		return com_error;
	
	/*if(buf[0] == 0x7F && buf[1] == '\0')
		return com_error;*/

	return buf[0];
}

void SCs_off(int mach_code)
{	
	if(mach_code > SCS_NUMB_MAX || mach_code < 1)
		return;
	if( SCs_fd[mach_code-1] >= 0 )
	{
		close(SCs_fd[mach_code-1]);
		SCs_fd[mach_code] = -1;
	}
}

