/** @file schard.h
 *  Abstraction layer over the SC systems hardware.
 */

#ifndef SCHARD_H_
#define SCHARD_H_

#include <stdio.h>
#include <time.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <termios.h>
#include <fcntl.h>

#include "sccommon.h"

/**
 * Initializes global hardware mechanisms.
 * @return 0 if successful
 */
int SC_hard_init();

/**
 * Initializes a SCslave module.
 * @param path the path o the serial port
 * @param mach_code modules id
 * @return 0 if successful else -1
 */
int SCs_init(const char* path, int mach_code );

/** Return values of function SCslave */
enum SCs_returns
{
	no_new_data = -1,	/**< no new data in module */
	com_error = -2,		/**< communication error */
	not_init = -3		/**< module not initialized */
};

/**
 * Gets data from a SCslave module.
 * @param mach_code modules id
 * @return if the value is below 0 then its one from SCs_returns, else its
 * the data from the SCslave module
 */
int SCs_data_cycle(int mach_code);

/**
 * Releases the SCslave module.
 * @param mach_code modules id
 */
void SCs_off(int mach_code);

#endif /* SCHARD_H_ */
