#include "wlctime.h"

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <assert.h>
 
 void dmscpy(struct dms* dst, const struct dms* src)
{
	dst->year = src->year;
	dst->month = src->month;
	dst->day = src->day;
}

struct dms* atodms(const char* isodate)
{
	struct dms* date = malloc(sizeof(struct dms));
	sscanf(isodate, "%u-%u-%u", &(date->year), &(date->month), &(date->day));
	return date;
}

char* print_dms(char* str, const struct dms* date)
{
	sprintf(str, "%u-%u-%u", date->year, date->month, date->day);
	return str;
}

int dms_res_dec(const struct dms* date)
{
	return date->year*1000 + date->month*100 + date->day;
}

int dms_greater(const struct dms* date1, const struct dms* date2)
{
	if(dms_res_dec(date1) > dms_res_dec(date2))
		return 1;
	else
		return 0;
}

int dms_less(const struct dms* date1, const struct dms* date0)
{
	if(dms_res_dec(date1) < dms_res_dec(date0))
		return 1;
	else
		return 0;
}

int dms_equal(const struct dms* date1, const struct dms* date0)
{
	if(dms_res_dec(date1) == dms_res_dec(date0))
		return 1;
	else
		return 0;
}

int inc_dms(struct dms* date)
{
	if(date->day == days_in_month(date->year, date->month))
	{
		date->day = 1;
		if(date->month == 12)
		{
			date->month = 1;
			date->year += 1;
		}
		else
			date->month += 1;
	}
	else
		date->day += 1;

	return 0;
}

int days_in_month(int year, int month)
{
	const static int days[12] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

	return days[month-1];
}

/******************************************************************************/

void tmscpy(struct tms* dst, const struct tms* src)
{
	dst->hour = src->hour;
	dst->min = src->min;
	dst->sec = src->sec;
}

struct tms* atotms(const char* isotime)
{
	struct tms* stime = malloc(sizeof(struct tms));
	sscanf(isotime, "%u:%u:%u", &(stime->hour),
		&(stime->min), &(stime->sec));
	return stime;
}

void atotms2(const char* isotime, struct tms* stime)
{
	sscanf(isotime, "%u:%u:%u", &(stime->hour), &(stime->min), &(stime->sec));
}

void tm_to_stm(struct tms* dst, const struct tm* src)
{
	dst->hour = src->tm_hour;
	dst->min = src->tm_min;
	dst->sec = src->tm_sec;
}

struct tms* difftms(const struct tms* time1, const struct tms* time0)
{
	struct tms* timeX = (struct tms*)malloc(sizeof(struct tms));
	const struct tms* timeE;
	
	if( tms_greater( time0, time1 ) )
	{
		tmscpy(timeX, time0);
		timeE = time1;
	}
	else
	{
		tmscpy(timeX, time1);
		timeE = time0;
	}

	timeX->hour -= timeE->hour;
	if(timeE->min > timeX->min)
	{
		timeX->hour -= 1;
		timeX->min += 60;
	}
	timeX->min -= timeE->min;
	if(timeE->sec > timeX->sec)
	{
		timeX->min -= 1;
		timeX->sec += 60;
	}
	
	timeX->sec -= timeE->sec;

	return timeX;
}

int tms_res_dec(const struct tms* time)
{
	return time->hour*1000 + time->min*100 + time->sec;
}

int tms_greater(const struct tms* time1, const struct tms* time0)
{
	if(tms_res_dec(time1) > tms_res_dec(time0))
		return 1;
	else
		return 0;
}

int tms_less(const struct tms* time1, const struct tms* time0)
{
	if(tms_res_dec(time1) < tms_res_dec(time0))
		return 1;
	else
		return 0;
}

int tms_equal(const struct tms* time1, const struct tms* time0)
{
	if(tms_res_dec(time1) == tms_res_dec(time0))
		return 1;
	else
		return 0;
}

int tms_to_sec(const struct tms* time)
{
	int r;
	assert(INT_MAX >= (59+59*60+23*3600) );
	r = time->sec + time->min*60 + time->hour*3600;
	return r;
}

char* print_tms(char* str, const struct tms* time)
{
	sprintf(str, "%u:%u:%u", time->hour, time->min, time->sec);
	return str;
}
