/** @file wlctime.h
 * Extends the time.h library.
 */

#ifndef WLCTIME_H_
#define WLCTIME_H_
 
#include "wlcstdlib.h"

#include <time.h>

/** 
 * Simple structure to manipulate dates.
 */
struct dms
{
	tiny_int	year, /**< years from 1900 */ 
			month, /**< month number */
			day; /**< day number */
};

/** Copys the date from src to dst. */
void dmscpy(struct dms* dst, const struct dms* src);
/**
 * Allocate a dms structure with date from string isodate.
 * @param isodate a string with a date in the ISO format
 * @return accolated dms structure
 */
struct dms* atodms(const char* isodate);
/** 
 * Prints the date from date to string str.
 * @param str date from date in ISO format
 * @param date source date
 * @return a pointer to str
 */
char* print_dms(char* str, const struct dms* date);

/**
 * Checks is date1 > date2
 * @return 1 if true else 0
 */
int dms_greater(const struct dms* date1, const struct dms* date2);
/**
* Checks is date1 > date2
* @return 1 if true else 0
*/
int dms_less(const struct dms* date1, const struct dms* date0);
/**
* Checks is date1 > date2
* @return 1 if true else 0
*/
int dms_equal(const struct dms* date1, const struct dms* date0);

/**
 * Increments date date.
 * @return -1 if date overlaps else 0
 */
int inc_dms(struct dms* date);

/**
 * @return the number of days in month month in year year
 */
int days_in_month(int year, int month);

/** Simple structure to manipulate time */
struct tms
{
	tiny_int hour; /**< hour */
	tiny_int min; /**< minutes */
	tiny_int sec; /**< secunds */
};

/** Copys time from src to dst */
void tmscpy(struct tms* dst, const struct tms* src);

/**
 * Allocates a tms structure with the time form string isotime.
 * @param isotime string with time in format HH:MM:SS
 * @return a pointer to tms structure with timw from isotime
 */
struct tms* atotms(const char* isotime);

/**
 * Copys the time from string isotime do stime.
 * @param isotime source time in format HH:MM:SS
 * @param stime destination
 */
void atotms2(const char* isotime, struct tms* stime);

/** Copys time from tm structure src to tms structure dst */
void tm_to_stm(struct tms* dst, const struct tm* src);

/**
 * @return diffrence between time1 and time0
 */
struct tms* difftms(const struct tms* time1, const struct tms* time0);

/**
 * Checks is time1 > time2
 * @return 1 if true else 0
 */
int tms_greater(const struct tms* time1, const struct tms* time0);

/**
 * Checks is time1 > time2
 * @return 1 if true else 0
 */
int tms_less(const struct tms* time1, const struct tms* time0);

/**
 * Checks is time1 > time2
 * @return 1 if true else 0
 */
int tms_equal(const struct tms* time1, const struct tms* time0);

/**
 * @return number of secunds from time 00:00:00 to time time
 */
int tms_to_sec(const struct tms* time);

/**
 * Prints time from time to string str.
 * @param str destination string
 * @param time source time to be printed
 * @return a pointer to str
 */
char* print_tms(char* str, const struct tms* time);

#endif /* WLCTIME_H_ */
