/*	Ver. 0.3
*/

use SCdb;

drop procedure if exists show_workers;

create procedure show_workers ()
select first_name, last_name from worker;

drop procedure if exists insert_worker;

create procedure insert_worker 
(in n_first_name varchar(30), in n_last_name varchar(30))
insert into worker (first_name, last_name)
values (n_first_name, n_last_name);

drop procedure if exists delete_worker;

delimiter //

create procedure delete_worker
(in n_first_name varchar(30), in n_last_name varchar(30))
begin
	declare Wid int;

	select id_worker into Wid from worker
	where n_first_name = first_name
	and n_last_name = last_name;

	delete from worker
	where id_worker = Wid;

	delete from work_shift
	where id_worker = Wid;
end; //

delimiter ;

drop procedure if exists insert_wl;

delimiter //

create procedure insert_wl (in n_wl_date date)
begin
	if
	(
		select id_wl from worklog as act
		where n_wl_date = act.wl_date
	) is null
	then
		insert into worklog (wl_date, trans_date)
		values (n_wl_date, curdate());
	end if;
end; //

delimiter ;

drop procedure if exists show_wl;

delimiter //

create procedure show_wl
(in Ayear int, in Amonth int)
begin
	select date_format(wl_date, '%e') from worklog
	where wl_date like concat(Ayear, '-', Amonth, '-%')
	or wl_date like concat(Ayear, '-0', Amonth, '-%');
end; //

delimiter ;

drop procedure if exists show_planed_shift;

create procedure show_planed_shift
(in Ayear int, in Amonth int)
select date_format(wl_date, '%e')
from work_shift join worklog using (id_wl)
where wl_date like concat(Ayear, '-', Amonth, '-%')
or wl_date like concat(Ayear, '-0', Amonth, '-%');



drop procedure if exists insert_SCdata;

create procedure insert_SCdata 
(in n_wl_date date, in n_stime time, in n_id_ws tinyint,  in n_input int)
insert into workstation values
(
	n_id_ws,
	(select id_wl from worklog where wl_date = n_wl_date),
	n_stime,
	n_input
);


drop procedure if exists insert_SCdata_override;

create procedure insert_SCdata_override
(in n_wl_date date, in n_stime time, in n_id_ws tinyint, in n_input int)
insert ignore into workstation values
(
	n_id_ws,
	(select id_wl from worklog where wl_date = n_wl_date),
	n_stime,
	n_input
);

drop procedure if exists delete_alldata;

delimiter //

create procedure delete_alldata
(in del_date date)
begin
	declare del_date_id int;
	select id_wl into del_date_id from worklog
	where wl_date = del_date;
	delete from workstation where id_ws = del_date_id;
	delete from work_shift where id_wl = del_date_id;
	delete from worklog where id_wl = del_date_id;
end; //

delimiter ;

drop procedure if exists insert_shift;

create procedure insert_shift
(in shift_date date, in worker_fname varchar(40),
in worker_lname varchar(40), in station int, in shift_num int)
insert into work_shift (id_worker, id_ws, id_wl, shift) values
(
	(
		select id_worker from worker as w
		where worker_fname = w.first_name
		and worker_lname = w.last_name
	),
	station,
	(
		select id_wl from worklog as w
		where shift_date = w.wl_date
	),
	shift_num
);

drop procedure if exists show_shift;

delimiter //

create procedure show_shift (in sdate date)
begin
	declare continue handler for sqlstate '02000' begin end;
	select first_name, last_name, id_ws, shift from
	(
		select * from
		(
			select * from work_shift join worker
			using (id_worker)
		) as A join worklog
		using (id_wl)
	) as B where sdate = B.wl_date;
end; //

delimiter ;

drop procedure if exists delete_shift;

delimiter //

create procedure delete_shift (in sdate date)
begin
	declare Tid int;

	select id_wl into Tid from worklog
	where wl_date = sdate;
	
	delete from work_shift
	where work_shift.id_wl = Tid;
end; //

delimiter ;

drop procedure if exists show_work;

delimiter //

create procedure show_work
(in n_first_name varchar(40), in n_last_name varchar(40),
in idate date)
begin
	declare Tid int;
	declare Wid int;
	declare continue handler for sqlstate '02000' begin end;

	select id_wl into Tid from worklog
	where wl_date = idate;
	
	select id_worker into Wid from worker
	where n_first_name = first_name
	and n_last_name = last_name;
	
	select id_ws, shift from work_shift
	where Tid = work_shift.id_wl
	and Wid = work_shift.id_worker;
end; //

delimiter ;

drop procedure if exists show_day_input;

delimiter //

create procedure show_day_input
(in sdate date, in tfrom time, in tto time, in ws int)
begin

	declare Tid int;
	declare continue handler for sqlstate '02000' begin end;
	declare continue handler for sqlstate 'HY000' begin end;

	select id_wl into Tid from worklog
	where wl_date = sdate;
	
	select input, stime from workstation
	where Tid = id_sdate
	and ws = id_ws
	and stime > tfrom
	and stime < tto;
end; //

delimiter ;

