#include "mainform.h"
#include "mainform.moc"

#include <QtGui/QLineEdit>
#include <QtGui/QSpinBox>
#include <QtGui/QTableWidget>
#include <QtGui/QGroupBox>
#include <QtGui/QComboBox>
#include <QtGui/QStatusBar>
#include <QtGui/QMessageBox>
#include <QtGui/QDialog>
#include <QtGui/QInputDialog>

#include "manage/scmanager_msg.h"
#include "manage/scmanage_common.h"

const QString MainForm::SCmanagerCmd =
#if defined(Q_OS_UNIX)
	"./scmanager"
#elif defined(Q_OS_WIN32)
	"scmanager.exe"
#endif
;

MainForm::MainForm()
	:
	check(":/icons/v.png"),
	nocheck(":/icons/x.png"),
	statusTimer(this)
{
	if( !QFileInfo(SCmanagerCmd).exists() )
	{
		QMessageBox::critical( this, "Fatal Error",
			QString("Brak backenda ") + SCmanagerCmd
			+ QString(" w katalogu roboczym") );
		qFatal("No backend");
	}
	
	ui.setupUi(this);
	
	planEdited = false;

	for( int i = 0; i < mainGrpsNumb; ++i )
		grpsBlock[i] = 0;

	ui.daysTable->setColumnWidth( 0, 60 );
	ui.daysTable->setColumnWidth( 1, 60 );

	// Seting up SCmanager process
	
	SCmanager_ws_save.setWorkingDirectory(QDir::currentPath());
	connect(&SCmanager_ws_save, SIGNAL(finished(int, QProcess::ExitStatus)),
        	this, SLOT(endSavePlan(int, QProcess::ExitStatus)));

	SCmanager_ws_ref.setWorkingDirectory(QDir::currentPath());
	connect(&SCmanager_ws_ref, SIGNAL(finished(int, QProcess::ExitStatus)),
        	this, SLOT(endRefreshPlan(int, QProcess::ExitStatus)));
	
	SCmanager_wr_add.setWorkingDirectory(QDir::currentPath());
	connect(&SCmanager_wr_add, SIGNAL(finished(int, QProcess::ExitStatus)),
        	this, SLOT(endAddWorker(int, QProcess::ExitStatus)));

	SCmanager_wr_del.setWorkingDirectory(QDir::currentPath());
	connect(&SCmanager_wr_del, SIGNAL(finished(int, QProcess::ExitStatus)),
        	this, SLOT(endDelWorker(int, QProcess::ExitStatus)));

	SCmanager_wr_ref.setWorkingDirectory(QDir::currentPath());
	connect(&SCmanager_wr_ref, SIGNAL(finished(int, QProcess::ExitStatus)),
        	this, SLOT(endRefreshWorkers(int, QProcess::ExitStatus)));

	SCmanager_wl_ref.setWorkingDirectory(QDir::currentPath());
	connect(&SCmanager_wl_ref, SIGNAL(finished(int, QProcess::ExitStatus)),
        	this, SLOT(endRefreshWls(int, QProcess::ExitStatus)));

	connect(&SCmanager_wl_del, SIGNAL(finished(int, QProcess::ExitStatus)),
        	this, SLOT(endDeleteData(int, QProcess::ExitStatus)));

	QDate now = QDate::currentDate();
	ui.yearSpinBox->setValue( now.year() );
	ui.monthSpinBox->setValue( now.month() );
	ui.deleteDataButton->setEnabled(false);
	ui.daysTable->clearSelection();

	// Connection UIs signals
	
	connect(&statusTimer, SIGNAL(timeout()),
		this, SLOT(refreshStatuss()));
	statusTimer.start(200);

	connect( ui.actionTransfer, SIGNAL(activated()),
		SLOT(makeTrans()) );
	connect( ui.actionRaport, SIGNAL(activated()),
		SLOT(generateRaport()) );
	connect( ui.actionAbout, SIGNAL(activated()),
		SLOT(aboutSCm()) );
	connect( ui.actionAboutQt, SIGNAL(activated()),
		qApp, SLOT(aboutQt()) );

	connectYearOrMonthChanged();
	connect( ui.daysTable, SIGNAL(cellPressed(int, int)),
		SLOT(dayChanged(int, int)) );
	connect( ui.deleteDataButton, SIGNAL(clicked()),
		SLOT(deleteData()) );
	
	connect( ui.planTable, SIGNAL(cellPressed(int, int)),
		SLOT(addWorkerToPlan(int, int)) );
	connect( ui.savePlanButton, SIGNAL(clicked()),
		SLOT(savePlan()) );
	connect( ui.cleanPlanButton, SIGNAL(clicked()),
		SLOT(cleanPlan()) );
	connect( ui.setClipboardButton, SIGNAL(clicked()),
		SLOT(setClipboard()) );
	connect( ui.getClipboardButton, SIGNAL(clicked()),
		SLOT(getClipboard()) );

	connect( ui.addWorkerButton, SIGNAL(clicked()),
		SLOT(addWorker()) );
	connect( ui.delWorkerButton, SIGNAL(clicked()),
		SLOT(delWorker()) );
	
	refreshWls();
	refreshWorkers();
	blockGroup( planGrp );
}

/* public slots ***************************************************************/

void MainForm::makeTrans()
{
	TransForm dialog(this);
	dialog.exec();
	refreshWls();
}

void MainForm::generateRaport()
{
	RaportForm dialog(this);
	dialog.exec();
}

void MainForm::manual()
{
	QMessageBox::information( this, "Information",
		"Opcja jeszcze nie zaimplementowana" );
}

void MainForm::aboutSCm()
{
	 QMessageBox::about( this, "O programie",
	 	"<b>SC Manage Center</b> jest "
	 	"Font-end'em opartym naplatformie Qt "
	 	"dla programu grupy zarządzającej systemu SC."
	 	"<br>Autor: Adam Walczak" );
}

void MainForm::yearOrMonthChanged()
{
	qDebug() << "yearOrMonthChanged::started";
	
	if(checkIfRestorePlan())
	{
		qDebug() << "yearOrMonthChanged::ended with restored plan";
		return;
	}
	
	if(!calendarDate.isNull())
	{
		calendarDate = QDate();
		blockGroup( planGrp );
	}
	
	ui.daysTable->clearSelection();
	
	ui.deleteDataButton->setEnabled(false);
	
	refreshWls();
	
	planEdited = false;
	
	qDebug() << "yearOrMonthChanged::ended";
}

void MainForm::dayChanged( int , int )
{
	qDebug() << "dayChanged::started";
	
	if(checkIfRestorePlan())
	{
		qDebug() << "dayChanged::ended with restored plan";
		return;
	}

	if(!ui.daysTable->item(ui.daysTable->currentRow(), 0))
	{
		ui.deleteDataButton->setEnabled(false);
		
		ui.planTable->clearContents();
		if(!calendarDate.isNull())
		{
			calendarDate = QDate();
			blockGroup( planGrp );
		}

		qDebug() << "dayChanged::ended with no plan";
		return;
	}

	if( calendarDate.isNull() )
		unblockGroup( planGrp );

	calendarDate.setDate( ui.yearSpinBox->value(),
		ui.monthSpinBox->value(),
		ui.daysTable->currentRow() + 1 );

	ui.deleteDataButton->setEnabled(true);

	refreshPlan();
		
	planEdited = false;

	qDebug() << "dayChanged::ended with new plan";
}

void MainForm::deleteData()
{
	qDebug() << "deleteData::started";
	
	blockGroup( planGrp );
	blockGroup( calendarGrp );
	
	QStringList argv;
	argv += SCM_CMD_DELETE_ALLDATA;
	argv += QString::number(ui.yearSpinBox->value()) + '-'
		+ QString::number(ui.monthSpinBox->value()) + '-'
		+ QString::number( calendarDate.day() );

	calendarDate = QDate();
	ui.planTable->clearContents();

	qDebug() << "deleteData::starting process:" << SCmanagerCmd
		<< argv.join(" ");

	SCmanager_wl_del.start(SCmanagerCmd, argv);
}

void MainForm::addWorkerToPlan( int row, int col )
{
	qDebug() << "addWorkerToPlan::started";
	
	if(!ui.planTable->isEnabled())
		return;
	planEdited = true;

	QTableWidgetItem *planItem = ui.planTable->currentItem();
	QListWidgetItem *workerItem = ui.workerList->currentItem();
	if(!workerItem)
	{
		qDebug() << "addWorkerToPlan::ended with null workerItem";
		return;
	}

	if(!planItem)
	{
		QTableWidgetItem *item = new QTableWidgetItem(
			workerItem->text() );
		ui.planTable->setItem( row, col, item );

		qDebug() << "addWorkerToPlan::ended with new planItem";
		return;
	}
	if( planItem->text() == workerItem->text())
	{
		delete planItem;
	}
	else
		planItem->setText( workerItem->text() );

	qDebug() << "addWorkerToPlan::ended with planItem value changed";
}

void MainForm::savePlan()
{
	qDebug() << "savePlan::started";
	
	blockGroup( planGrp );
	blockGroup( calendarGrp );
	
	QStringList argv;
	argv += SCM_CMD_INSERT_SHIFT;
	argv += QString::number(ui.yearSpinBox->value()) + '-'
		+ QString::number(ui.monthSpinBox->value()) + '-'
		+ QString::number( calendarDate.day() );

	qDebug() << "savePlan::starting process:" << SCmanagerCmd
		<< argv.join(" ");

	SCmanager_ws_save.start(SCmanagerCmd, argv);

	qDebug() << "savePlan::writing to stdin";
	
	QString lineArg;
	for(int i = 0; i < ui.planTable->rowCount(); ++i)
		for(int j = 0; j < ui.planTable->columnCount(); ++j)
			if( ui.planTable->item(i, j) )
			{
				lineArg = "";
				QTextStream(&lineArg)
					<< ui.planTable->item(i, j)->text()
					<< ' '
					<< QString::number(i+1) << ' '
					<< QString::number(j+1) << '\n';
				SCmanager_ws_save.write(lineArg.toUtf8());
			}

	qDebug() << "savePlan::closing stdin";
	SCmanager_ws_save.closeWriteChannel();
}

void MainForm::cleanPlan()
{
	ui.planTable->clearContents();
	ui.planTable->setFocus();
	
	planEdited = true;
}

void MainForm::setClipboard()
{
	qDebug() << "setClipboard::started";
	
	for(int i = 0; i < SCS_NUMB_MAX; ++i)
		for(int j = 0; j < shifts_num; ++j)
		{
			QTableWidgetItem *item = ui.planTable->item( i, j );
			if( item )
				clipboard[i][j] = item->text();
			else
				clipboard[i][j] = QString::Null();
		}
	
	ui.clipboardDateLineEdit->setText(
		calendarDate.toString( Qt::ISODate ) );

	qDebug() << "setClipboard::ended";
}

void MainForm::getClipboard()
{
	qDebug() << "getClipboard::started";

	cleanPlan();
	for(int i = 0; i < SCS_NUMB_MAX; ++i)
		for(int j = 0; j < shifts_num; ++j)
			if( !clipboard[i][j].isNull() )
			{
				QTableWidgetItem *item = new QTableWidgetItem(
					clipboard[i][j] );
				ui.planTable->setItem( i, j, item );
			}

	planEdited = true;

	qDebug() << "getClipboard::ended";
}

void MainForm::addWorker()
{
	qDebug() << "addWorker::started";

	blockGroup( workersGrp );
	
	bool ok;
	QString name = QInputDialog::getText(this, "Dodaj pracownika",
		"Podaj imię i nazwisko:", QLineEdit::Normal, QString::null,
		&ok);
		
	if(!ok)
	{
		qDebug() << "addWorker::aborted";
		return;
	}
	
	int space = name.indexOf(' ');
	if(space == -1)
	{
		qCritical() << "addWorker::no last name";
		
		QMessageBox::critical(this, "Error", "Nie podałeś nazwiska");
		return;
	}
	
	QStringList argv;
	argv += SCM_CMD_INSERT_WORKER;

	qDebug() << "addWorker::starting process:" << SCmanagerCmd
		<< argv.join(" ");

	SCmanager_wr_add.start(SCmanagerCmd, argv);
	SCmanager_wr_add.write( QString(name + QChar('\n')).toUtf8() );
}

void MainForm::delWorker()
{
	qDebug() << "delWorker::started";

	blockGroup( workersGrp );
	blockGroup( planGrp );
	blockGroup( calendarGrp );

	QString name = ui.workerList->currentItem()->text();
	
	QStringList argv;
	argv += SCM_CMD_DELETE_WORKER;
	argv += name.split(' ');

	qDebug() << "delWorker::starting process:" << SCmanagerCmd
		<< argv.join(" ");

	SCmanager_wr_del.start(SCmanagerCmd, argv);
}

void MainForm::refreshStatuss()
{
	QString statusText;
	if(SCmanager_ws_save.state() == QProcess::Running)
		statusText += "[Zapisywanie planu]";
	if(SCmanager_ws_ref.state() == QProcess::Running)
		statusText += "[Wczytywanie planu]";
	if(SCmanager_wr_add.state() == QProcess::Running)
		statusText += "[Zapisywanie pracownika]";
	if(SCmanager_wr_del.state() == QProcess::Running)
		statusText += "[Usuwanie pracownika]";
	if(SCmanager_wr_ref.state() == QProcess::Running)
		statusText += "[Wczytywanie pracowników]";
	if(SCmanager_wl_ref.state() == QProcess::Running)
		statusText += "[Wczytywanie kalendarza]";
	this->statusBar()->showMessage(statusText);
}

void MainForm::refreshWls()
{
	qDebug() << "refreshWls::started";

	blockGroup( calendarGrp );
	blockGroup( planGrp );

	QStringList argv;
	argv += SCM_CMD_GET_WLS;
	argv += QString::number(ui.yearSpinBox->value());
	argv += QString::number(ui.monthSpinBox->value());

	qDebug() << "refreshWls::starting process:" << SCmanagerCmd
		<< argv.join(" ");
	
	SCmanager_wl_ref.start( SCmanagerCmd , argv );
}

void MainForm::refreshWorkers()
{
	qDebug() << "refreshWorkers::started";

	blockGroup( workersGrp );

	QStringList argv;
	argv += SCM_CMD_GET_WORKERS;

	qDebug() << "refreshWorkers::starting process:"
		<< SCmanagerCmd << argv.join(" ");

	SCmanager_wr_ref.start(SCmanagerCmd, argv);
}

void MainForm::refreshPlan()
{
	qDebug() << "refreshPlan::started";

	blockGroup( planGrp );

	QStringList argv;
	argv += SCM_CMD_GET_SHIFT;
	argv += calendarDate.toString(Qt::ISODate);

	qDebug() << "refreshPlan::starting process:" << SCmanagerCmd
		<< argv.join(" ");
	
	SCmanager_ws_ref.start(SCmanagerCmd, argv);
}

void MainForm::endDeleteData( int exitCode, QProcess::ExitStatus exitStatus )
{
	if( exitCode != 0 || exitStatus != QProcess::NormalExit )
	{
		qCritical() << "deleteData::ended with error";
		
		QMessageBox::critical( this, "Error",
			"Nie można usunąć danych.\n"
			+ QString( SCmanager_wl_ref.readAllStandardError() ) );
	}

	unblockGroup( calendarGrp );

	ui.daysTable->clearSelection();

	refreshWls();
}

void MainForm::endRefreshWls( int exitCode, QProcess::ExitStatus exitStatus )
{
	if( exitCode != 0 || exitStatus != QProcess::NormalExit )
	{
		qCritical() << "refreshWls::ended with error";
		
		QMessageBox::critical( this, "Error",
			"Nie można odświeżyć kalendarza.\n"
			+ QString( SCmanager_wl_ref.readAllStandardError() ) );
	}
	else
	{
		ui.daysTable->clearContents();
		
		QString strday;
		while(SCmanager_wl_ref.canReadLine())
		{
			strday = SCmanager_wl_ref.readLine().simplified();
			if(strday == SCM_SEPARATOR)
				break;
			QTableWidgetItem *vitem = new QTableWidgetItem( check,
				QString::Null() );
			QTableWidgetItem *xitem = new QTableWidgetItem( nocheck,
				QString::Null() );
			ui.daysTable->setItem( strday.toShort() - 1, 0, vitem );
			ui.daysTable->setItem( strday.toShort() - 1, 1, xitem );
			
		}
		while(SCmanager_wl_ref.canReadLine())
		{
			strday = SCmanager_wl_ref.readLine().simplified();
			QTableWidgetItem *item = new QTableWidgetItem( check,
				QString::Null() );
			ui.daysTable->setItem( strday.toShort() - 1, 1, item );
		}

		qDebug() << "refreshWls::ended succesfull";
	}

	unblockGroup( calendarGrp );
	unblockGroup( planGrp );
}

void MainForm::endRefreshWorkers( int exitCode,
	QProcess::ExitStatus exitStatus )
{
	if( exitStatus != 0 || exitStatus != QProcess::NormalExit )
	{
		qCritical() << "refreshWorkers::ended with error";
		
		QMessageBox::critical(this, "Error",
			"Nie można pobrać wszystkich danych o pracownikach.\n"
			+ QString(SCmanager_wr_ref.readAllStandardError()));
	}
	else
	{
		ui.workerList->clear();
		while(SCmanager_wr_ref.canReadLine())
		{
			QString name = SCmanager_wr_ref.readLine().simplified();
			ui.workerList->addItem(name);
		}
		ui.workerList->sortItems();

		qDebug() << "refreshWorkers::ended succesfull";
	}

	unblockGroup( workersGrp );
}

void MainForm::endRefreshPlan( int exitCode, QProcess::ExitStatus exitStatus )
{
	if( exitStatus != 0 || exitStatus != QProcess::NormalExit )
	{
		qCritical() << "refreshPlan::ended with error";
		
		QMessageBox::critical(this, "Error",
			QString("Nie udało się odczytać całego planu.\n")
			+ QString(SCmanager_ws_ref.readAllStandardError()));
	}
	else
	{
		ui.planTable->clearContents();
	
		QString lineArg, first_name, last_name;
		int station, shift;
		while(SCmanager_ws_ref.canReadLine())
		{
			lineArg = SCmanager_ws_ref.readLine().simplified();
			QTextStream(&lineArg) >> first_name >> last_name
				>> station >> shift;
			QTableWidgetItem *item = new QTableWidgetItem(
				first_name
				+ QChar(' ')
				+ last_name );
			ui.planTable->setItem(station - 1, shift - 1, item );
		}

		qDebug() << "refreshPlan::ended succesfull";
	}

	unblockGroup( planGrp );
}

void MainForm::endSavePlan( int exitCode, QProcess::ExitStatus exitStatus )
{
	if( exitCode != 0 || exitStatus != QProcess::NormalExit )
	{
		qCritical() << "savePlan ended with error";
		
		QMessageBox::critical(this, "Error",
			QString("Nie udało się zapisać planu.\n")
			+ QString(SCmanager_ws_save.readAllStandardError()));
	}
	else
	{
		qDebug() << "savePlan ended succesfull";
		
		planEdited = false;
	}

	refreshWls();
	refreshPlan();
	
	unblockGroup( calendarGrp );
	unblockGroup( planGrp );

	ui.daysTable->selectRow( calendarDate.day() - 1 );
}

void MainForm::endAddWorker( int exitCode, QProcess::ExitStatus exitStatus )
{
	if( exitCode != 0 || exitStatus != QProcess::NormalExit )
	{
		qCritical() << "addWorker::ended with error";
		
		QMessageBox::critical( this, "Error",
			"Nie można dodać pracownika.\n"
			+ QString(SCmanager_wr_add.readAllStandardError()) );
	}
	else
		qDebug() << "addWorker::ended succesfull";

	unblockGroup( workersGrp );

	refreshWorkers();
}

void MainForm::endDelWorker( int exitCode, QProcess::ExitStatus exitStatus )
{
	if( exitCode != 0 || exitStatus != QProcess::NormalExit )
	{
		qCritical() << "delWorker::ended with error";
		
		QMessageBox::critical( this, "Error",
			QString("Usuwanie pracownika nie powiodło się.\n")
			+ QString(SCmanager_wr_del.readAllStandardError()) );
	}
	else
		qDebug() << "delWorker::ended succesfull";

	unblockGroup( workersGrp );
	unblockGroup( planGrp );
	unblockGroup( calendarGrp );
	
	refreshWorkers();
	refreshPlan();
}

/* private ********************************************************************/

void MainForm::blockGroup( mainGrps grp )
{
	Q_ASSERT(grp != mainGrpsNumb);

	if( grpsBlock[grp] == 0 )
	{
		switch(grp)
		{
			case calendarGrp :
			{
				daySelection = ui.daysTable->currentRow();
				ui.calendarGroupBox->setEnabled(false);
			}	break;
			case planGrp :
				ui.planGroupBox->setEnabled(false);
				break;
			case workersGrp :
				ui.workersGroupBox->setEnabled(false);
				break;
		}
	}

	++grpsBlock[grp];
	qDebug() << "blockGroup::state" << grp << "=" << grpsBlock[grp];
}

void MainForm::unblockGroup( mainGrps grp )
{
	Q_ASSERT(grp != mainGrpsNumb);

	if( grpsBlock[grp] == 1 )
	{
		switch(grp)
		{
			case calendarGrp :
			{
				ui.calendarGroupBox->setEnabled(true);
				ui.daysTable->setCurrentCell(daySelection, 0);
			}	break;
			case planGrp :
				ui.planGroupBox->setEnabled(true);
				break;
			case workersGrp :
				ui.workersGroupBox->setEnabled(true);
				break;
		}
	}

	--grpsBlock[grp];
	qDebug() << "unblockGroup::state" << grp << "=" << grpsBlock[grp];
}

void MainForm::connectYearOrMonthChanged()
{
	connect(ui.yearSpinBox, SIGNAL(valueChanged(int)),
		SLOT(yearOrMonthChanged()));
	connect(ui.monthSpinBox, SIGNAL(valueChanged(int)),
		SLOT(yearOrMonthChanged()));
}

void MainForm::disconnectYearOrMonthChanged()
{
	disconnect(ui.yearSpinBox, SIGNAL(valueChanged(int)),
		this, SLOT(yearOrMonthChanged()));
	disconnect(ui.monthSpinBox, SIGNAL(valueChanged(int)),
		this, SLOT(yearOrMonthChanged()));
}

bool MainForm::checkIfRestorePlan()
{
	if(planEdited)
	{
		if( QMessageBox::question(this, "Zmiana planu",
			"Czy napewno chcesz porzucić zmiany w obecnym "
			"planie dnia ?",
			QMessageBox::Yes, QMessageBox::No,
			QMessageBox::NoButton) ==  QMessageBox::No )
		{
			disconnectYearOrMonthChanged();
			
			ui.yearSpinBox->setValue(calendarDate.year());
			ui.monthSpinBox->setValue(calendarDate.month());
			if(calendarDate.day() == 0)
				ui.daysTable->clearSelection();
			else
				ui.daysTable->selectRow(
					calendarDate.day() - 1 );

			connectYearOrMonthChanged();
			return true;
		}
		else
		{
			planEdited = false;
			return false;
		}
	}
	else
		return false;
}
