#include "raportform.h"
#include "raportform.moc"

#include <QtGui/QFileDialog>
#include <QtGui/QRadioButton>
#include <QtGui/QLineEdit>
#include <QtGui/QLabel>
#include <QtGui/QSpinBox>
#include <QtGui/QToolBox>

#include "sccommon.h"
#include "manage/scraport_msg.h"

const QString RaportForm::SCraportCmd =
#if defined(Q_OS_UNIX)
	"./scraport"
#elif defined(Q_OS_WIN32)
	"scraport.exe"
#endif
;

RaportForm::RaportForm(QWidget *parent)
	:
	QDialog(parent),
	SCraport(this),
	raport(this),
	generating( QMessageBox::Information, "Generacja raportu",
		"Trwa generacja raportu. Proszę czekać...",
		QMessageBox::NoButton, this)
{
	ui.setupUi(this);
	QDate now = QDate::currentDate();

	button = new QPushButton("OK", this);
	generating.addButton( button, QMessageBox::AcceptRole );
	button->setEnabled(false);
	connect( button, SIGNAL(clicked()),
		SLOT(accept()) );

	connect( ui.buttonBox, SIGNAL(accepted()),
		SLOT(startGeneration()) );
	connect( ui.buttonBox, SIGNAL(rejected()),
		SLOT(reject()) );

	ui.oneDayDate->setDate(now);
	ui.oneMonthDate->setDate(now);
	ui.shortDate->setDate(now);
	ui.wsSpinBox->setMaximum(SCS_NUMB_MAX);
	
	connect(&SCraport, SIGNAL(finished(int, QProcess::ExitStatus)),
        	SLOT(endGeneration(int, QProcess::ExitStatus)));

	SCraport.setWorkingDirectory(QDir::currentPath());
}

void RaportForm::startGeneration()
{
	qDebug() << "RaportForm::starting generation";
	
	QFileDialog fileDialog( this, "Zapis raport jako" );
	fileDialog.setFileMode(QFileDialog::AnyFile);
	QStringList filters;
	QString	htmlFilter = "Strona HTML (*.html)",
		txtFilter = "Plik tekstowy (*.txt)";
	filters << htmlFilter
		<< txtFilter;
	fileDialog.setFilters(filters);
	fileDialog.setDirectory( QDir::homePath() );
	if( fileDialog.exec() != QDialog::Accepted )
	{
		qDebug() << "RaportForm::ended by user";
		return;
	}

	QString fileSuffix;
	const QString selectedFilter = fileDialog.selectedFilter();
	if( selectedFilter == htmlFilter )
		fileSuffix = ".html";
	else if( selectedFilter == txtFilter )
		fileSuffix = ".txt";

	QString fileName = fileDialog.selectedFiles().first();
	if( !fileName.endsWith( fileSuffix ) )
		fileName += fileSuffix;
	raport.setFileName( fileName );
	if(!raport.open(QIODevice::WriteOnly))
	{
		QMessageBox::critical(this, "Error",
			"Nie można utworzyć pliku" );

		qCritical() << "RaportForm::ended becouse couldnt create file";
		return;
	}

	generating.show();

	QStringList argv;
	switch(ui.raportToolBox->currentIndex())
	{
		case oneDay:
		{
			argv	<< SCM_CMD_GEN_DAY_RAPORT
				<< ui.oneDayDate->date().toString(Qt::ISODate);
			
		}	break;
		case oneMonth:
		{
			argv	<< SCM_CMD_GEN_MONTH_RAPORT
				<< QString::number(
					ui.oneMonthDate->date().year() )
				<< QString::number(
					ui.oneMonthDate->date().month() );
		}	break;
		case shortWorkstation:
		{
			argv	<< SCM_CMD_GEN_SHORT_RAPORT
			<< ui.shortDate->date().toString(Qt::ISODate)
			<< ui.shortFromTime->time().toString(Qt::ISODate)
			<< ui.shortToTime->time().toString(Qt::ISODate)
			<< QString::number( ui.wsSpinBox->value() );
		}	break;
		default :
		{
			qCritical() << "RaportForm::ended becouse of wrong"
				<< "raport type index";
			return;
		}
	}

	if( selectedFilter == htmlFilter )
		argv << SCM_FORMAT_HTML;
	else if( selectedFilter == txtFilter )
		argv << SCM_FORMAT_TXT;
	
	SCraport.start( SCraportCmd, argv );
}

void RaportForm::endGeneration( int exitCode,
	QProcess::ExitStatus exitStatus )
{
	raport.write(SCraport.readAllStandardOutput());
	raport.close();
	
	if( exitCode != 0 || exitStatus != QProcess::NormalExit )
	{
		generating.setIcon(QMessageBox::Critical);
		generating.setText(
			QString("Powstały błędy. podczas generowania \
				raportu.\n")
			+ QString(SCraport.readAllStandardError()) );
		qCritical() << "RaportForm::ended with errors";
	}
	else
	{
		generating.setText("Generacja zakończyła się sukcesem");
		qDebug() << "RaportForm::ended succesfull";
	}

	button->setEnabled(true);
}
