#include "transform.h"
#include "transform.moc"

#include <QtGui/QLabel>
#include <QtGui/QCheckBox>
#include <QtGui/QPushButton>
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>

#include "manage/scmanage_common.h"

const QString TransForm::SCwl2dbCmd =
#if defined(Q_OS_UNIX)
	"./scwl2db"
#elif defined(Q_OS_WIN32)
	"scwl2db.exe"
#endif
;

TransForm::TransForm(QWidget *parent)
    :
    QDialog(parent),
    SCwl2db(this)
{
	ui.setupUi(this);

	connect( ui.transDirButton, SIGNAL(clicked()),
		SLOT(setTransDir()) );
	connect(&SCwl2db, SIGNAL(finished(int, QProcess::ExitStatus)),
		SLOT(endTrans(int, QProcess::ExitStatus)));
	SCwl2db.setWorkingDirectory(QDir::currentPath());
	connect( ui.startButton, SIGNAL(clicked()),
		SLOT(startTrans()) );

	ui.transDirEdit->setText( QDir::homePath() );
}

void TransForm::setTransDir()
{
	QString dir = QFileDialog::getExistingDirectory( this,
		"Katalog transferowy", ui.transDirEdit->text() );
	if( !dir.isNull() )
		ui.transDirEdit->setText(dir);
}

void TransForm::startTrans()
{
	qDebug() << "TransForm::starting transfer";
	
	ui.startButton->setEnabled(false);

	QString transDir;
	if( ui.transDirGroupBox->isChecked() )
		transDir = ui.transDirEdit->text();
		
	QStringList argv;
	argv	<< (ui.eraseCheckBox->isChecked() ? "1" : "0")
		<< (ui.overCheckBox->isChecked() ? "1" : "0");

	if( !transDir.isNull() )
		argv << transDir;

	qDebug() << "TransForm::starting process:"
		<< SCwl2dbCmd << argv.join(" ");
	
	SCwl2db.start( SCwl2dbCmd, argv );
}

void TransForm::endTrans( int exitCode, QProcess::ExitStatus exitStatus )
{
	QString buf;
	while( SCwl2db.canReadLine() )
	{
		buf = SCwl2db.readLine().simplified();
		if(buf == SCM_SEPARATOR)
			break;
		ui.wlList->addItem(buf);
	}
	
	QString date, time, type, code;
	int rows = 0;
	
	while( SCwl2db.canReadLine() )
	{
		buf = SCwl2db.readLine().simplified();
		QTextStream(&buf) >> date >> time >> type >> code;
		ui.errorTable->setRowCount(++rows);
		QTableWidgetItem *item;
		item = new QTableWidgetItem(date);
		ui.errorTable->setItem( rows - 1, 0, item );
		item = new QTableWidgetItem(time);
		ui.errorTable->setItem( rows - 1, 1, item );
		item = new QTableWidgetItem(type);
		ui.errorTable->setItem( rows - 1, 2, item );
		item = new QTableWidgetItem(code);
		ui.errorTable->setItem( rows - 1, 3, item );
	}
	
	if( exitCode != 0 || exitStatus != QProcess::NormalExit )
	{
		QString errors = SCwl2db.readAllStandardError();
		QMessageBox::critical(this, "Błędy podczas transferu", errors);

		qCritical() << "TransForm::ended with errors";
	}

	qDebug() << "TransForm::ended succesfull";
	
	ui.startButton->setEnabled(true);
}
