/*
 * scmanager - program to manage data on the SCdb database
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "scdata.h"
#include "analis/scdata_a.h"
#include "manage/scdb.h"
#include "manage/scmanager_msg.h"
#include "manage/scmanage_common.h"

#include "wlcstdc.h"

#include "iniparser.h"

struct SCmanager_cfg
{
	struct SCdb_cfg *db_cfg;
} cfg;

int init_cfg_and_SCdb();
void free_cfg_and_SCdb();

int get_wls(int year, int month);

int get_workers();
int insert_worker();
int delete_worker(const char* first_name, const char* last_name);

int insert_shift(const char* date);
int get_shift(const char* date);

int delete_alldata(const char* date);

int main(int argc, char** argv)
{
	int res;

	atexit(free_cfg_and_SCdb);

	if(argc < 2)
	{
		fputs("Za mała liczba argumentów\n", stderr);
		return exit_err_argc;
	}

	res = init_cfg_and_SCdb();
	
	switch(res)
	{
		case exit_err_ini : fputs("Plik konfiguracyjny nie zawiera "
			"niezbędnych danych\n", stderr);
			break;
		case exit_err_db : fputs("Błąd przy inicjalizacji dazy danych",
			stderr);
			break;
		case 0 :
		{
			if(strcmp(argv[1], SCM_CMD_GET_WLS) == 0)
				if(argc == 4)
					res = get_wls(atoi(argv[2]),
						atoi(argv[3]));
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_INSERT_WORKER) == 0)
				if(argc == 2)
					res = insert_worker();
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_GET_WORKERS) == 0)
				if(argc == 2)
					res = get_workers();
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_DELETE_WORKER) == 0)
				if(argc == 4)
					res = delete_worker(argv[2], argv[3]);
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_INSERT_SHIFT) == 0)
				if(argc == 3)
					res = insert_shift(argv[2]);
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_GET_SHIFT) == 0)
				if(argc == 3)
					res = get_shift(argv[2]);
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_DELETE_ALLDATA) == 0)
				if(argc == 3)
					res = delete_alldata(argv[2]);
				else
					res = exit_err_argc;
			else
			{
				res = exit_err_cmd;
				fputs("Zly opcja\n", stderr);
			}
			
			if(res == exit_err_argc)
				fputs("Zla ilosc argumentow\n", stderr);
		}	break;
		default : fputs( "Nieznany błąd", stderr );
	}
	
	/*free_cfg_and_SCdb(); is in atexit()*/
	return res;
}

int init_cfg_and_SCdb()
{
	cfg.db_cfg = make_SCdb_cfg("scdb.ini");
	if( !cfg.db_cfg )
		return exit_err_ini;

	if( SCdb_init(cfg.db_cfg) != 0 )
		return exit_err_db;

	return 0;
}

void free_cfg_and_SCdb()
{
	/*iniparser_freedict(cfg.d);*/
	SCdb_free();
	fflush(stdout);
}

int get_wls(int year, int month)
{
	SCDB_RES *res;
	SCDB_ROW row;

	if( SCdb_get_wls(year, month) != 0 )
		return exit_err_db;

	res = SCdb_store_result();

	while ((row = SCdb_fetch_row(res)) != NULL)
		printf("%s\n", row[0]);

	SCdb_free_result(res);

	SCdb_reset_connection(cfg.db_cfg);

	puts(SCM_SEPARATOR);

	if( SCdb_get_planed_shift(year, month) != 0 )
		return exit_err_db;

	res = SCdb_store_result();

	while((row = SCdb_fetch_row(res)) != NULL)
		printf("%s\n", row[0]);

	SCdb_free_result(res);
	
	return 0;
}

int get_planed_shift(int year, int month)
{
	SCDB_RES *res;
	SCDB_ROW row;

	if ( SCdb_get_planed_shift( year, month ) != 0 )
		return exit_err_db;

	res = SCdb_store_result();

	while ((row = SCdb_fetch_row(res)) != NULL)
		printf("%s\n", row[0]);

	SCdb_free_result(res);
	
	return 0;
}

int get_workers()
{
	SCDB_RES *res;
	SCDB_ROW row;

	if ( SCdb_get_workers() != 0 )
		return exit_err_db;

	res = SCdb_store_result();

	while ((row = SCdb_fetch_row(res)) != NULL)
		printf("%s %s\n", row[0], row[1]);

	SCdb_free_result(res);
	
	return 0;
}

int insert_worker()
{
	const char first_name[40], last_name[40];
	scanf("%s %s", first_name, last_name);
	if( SCdb_insert_worker(first_name, last_name) != 0 )
		return exit_err_db;

	return 0;
}

int delete_worker(const char* first_name, const char* last_name)
{
	if( SCdb_delete_worker(first_name, last_name) != 0 )
		return exit_err_db;

	return 0;
}

int insert_shift(const char* date)
{
	char	strb[100],
		first_name[50],
		last_name[50];
	int station, shift;

	SCdb_delete_shift(date);
	if( SCdb_delete_shift(date) != 0 )
		return exit_err_db;
	
	fgets(strb, 50, stdin);
	while(!feof(stdin))
	{
		sscanf(strb, "%s %s %u %u", first_name, last_name,
			&station, &shift);

		if( SCdb_insert_shift(date, first_name, last_name,
			station, shift) != 0)
			return exit_err_db;
		fgets(strb, 50, stdin);
	}

	return 0;
}

int get_shift(const char* date)
{
	SCDB_RES *res;
	SCDB_ROW row;
	
	if( SCdb_get_shift(date) != 0)
		return exit_err_db;
	
	res = SCdb_store_result();

	while ((row = SCdb_fetch_row(res)) != NULL)
		printf("%s %s %s %s\n", row[0], row[1], row[2], row[3]);

	SCdb_free_result(res);
	
	return 0;
}

int delete_alldata(const char* date)
{
	if( SCdb_delete_alldata(date) != 0)
		return exit_err_db;
	
	return 0;
}
