/*
 * scraport - generates reports using based on data gathered on SCdb
 */

#include <stdlib.h>
#include <stdio.h>

#include "scdata.h"
#include "analis/scdata_a.h"
#include "manage/scdb.h"
#include "manage/scmanage_common.h"
#include "manage/scraport_msg.h"

#include "wlcstdc.h"

#include "iniparser.h"

struct SCraport_cfg
{
	dictionary *d;
	char	*fstart, *fend,
		*sstart, *send,
		*tstart, *tend;
	char** values;
	struct SCdb_cfg *db_cfg;
} cfg;

int init_cfg_and_SCdb();
void free_cfg_and_SCdb();

int generate_day_raport(const char* sdate, const char* strformat);
int generate_month_raport(int year, int month, const char* strformat);
int generate_short_raport(const char* sdate, const char* from_strtime,
	const char* to_strtime, int ws, const char* strformat);

int main( int argc, char **argv)
{
	int res;

	atexit(free_cfg_and_SCdb);

	if(argc < 2)
	{
		fputs("Za mała liczba argumentów\n", stderr);
		return exit_err_argc;
	}

	res = init_cfg_and_SCdb();
	
	switch(res)
	{
		case exit_err_ini : fputs("Plik konfiguracyjny nie zawiera "
			"niezbędnych danych\n", stderr);
			break;
		case exit_err_db : fputs("Błąd przy inicjalizacji dazy danych",
			stderr);
			break;
		case 0 :
		{
			if(strcmp(argv[1], SCM_CMD_GEN_DAY_RAPORT) == 0)
				if(argc == 4)
					res = generate_day_raport(argv[2],
						argv[3]);
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_GEN_MONTH_RAPORT) == 0)
				if(argc == 5)
					res = generate_month_raport(
						atoi(argv[2]), atoi(argv[3]),
						argv[4]);
				else
					res = exit_err_argc;
			else if(strcmp(argv[1], SCM_CMD_GEN_SHORT_RAPORT) == 0)
				if(argc == 7)
					res = generate_short_raport(argv[2],
						argv[3], argv[4],
						atoi(argv[5]), argv[6]);
				else
					res = exit_err_argc;
			else
			{
				res = exit_err_cmd;
				fputs("Zly opcja\n", stderr);
			}
			
			if(res == exit_err_argc)
				fputs("Zla ilosc argumentow\n", stderr);
		}	break;
		default : fputs( "Nieznany błąd", stderr );
	}
	
	/*free_cfg_and_SCdb(); is in atexit()*/
	return res;
}

int init_cfg_and_SCdb()
{
	int i;
	char strbufA[100];
	
	cfg.d = iniparser_load("scraport.ini");
	if(!cfg.d)
		return exit_err_ini;

	cfg.fstart = iniparser_getstr(cfg.d, "shift:first_start");
	if(!cfg.fstart)
		return exit_err_ini;
	cfg.fend = iniparser_getstr(cfg.d, "shift:first_end");
	if(!cfg.fend)
		return exit_err_ini;
	cfg.sstart = iniparser_getstr(cfg.d, "shift:second_start");
	if(!cfg.sstart)
		return exit_err_ini;
	cfg.send = iniparser_getstr(cfg.d, "shift:second_end");
	if(!cfg.send)
		return exit_err_ini;
	cfg.tstart = iniparser_getstr(cfg.d, "shift:third_start");
	if(!cfg.tstart)
		return exit_err_ini;
	cfg.tend = iniparser_getstr(cfg.d, "shift:third_end");
	if(!cfg.tend)
		return exit_err_ini;

	cfg.values = malloc(SC_score_numb*sizeof(char*));
	for(i = 0; i < SC_score_numb; i += 1)
	{
		sprintf(strbufA, "values:%i", i);
		cfg.values[i] = iniparser_getstr(cfg.d, strbufA);
		if(cfg.values[i] == NULL)
		{
			cfg.values[i] = malloc(sizeof(char)*30);
			sprintf(cfg.values[i], "undef %i", i);
		}
	}

	cfg.db_cfg = make_SCdb_cfg("scdb.ini");
	if( !cfg.db_cfg )
		return exit_err_ini;

	if( SCdb_init(cfg.db_cfg) != 0 )
		return exit_err_db;

	return 0;
}

void free_cfg_and_SCdb()
{
	iniparser_freedict(cfg.d);
	SCdb_free();
	fflush(stdout);
}

enum format_id
{
	txt,
	html
};

void rap_print_header(int format, const char* title, const char* sdate)
{
	switch(format)
	{
		case txt : printf("Typ raportu: %s\nData: %s\n\t",
			title, sdate); break;
		case html :
		{
			printf( "<html>\n<head>\n<title>SCmanager raport\
				</title>\n<meta HTTP-EQUIV=\"Content-type\" \
				CONTENT=\"text/html; charset=utf8\">\n</head>\
				\n<body>\n<h4>Typ raportu: %s.</h4>", title );
			printf("<h4>Data: %s</h4>\n", sdate);
			puts("<table border=\"1\" cellspacing=\"0\" \
				cellpadding=\"3\">\n<tr>\n<td  width=\"200\">\
				</td>");
		}	break;
	}
}

void rap_print_first_row(int format)
{
	int i;
	for(i = 0; i < SC_score_numb; i += 1)
		switch(format)
		{
			case txt : printf("\t%s", cfg.values[i]); break;
			case html: printf("<td  width=\"60\">%s</td>",
				cfg.values[i]); break;
		}
	switch(format)
	{
		case txt : putchar('\n'); break;
		case html : puts("\n</tr>"); break;
	}
}

void rap_print_first_col(int format, const char* first_name,
	const char* last_name)
{
	switch(format)
		{
			case txt : printf("%s %s", first_name, last_name);
				break;
			case html : printf("<tr>\n<td>%s %s</td>",
				first_name, last_name); break;
		}
}

void rap_print_score_col(int format, int* score)
{
	int i;
	for(i = 0; i < SC_score_numb; i += 1)
		switch(format)
		{
			case txt : printf("\t%u", score[i]); break;
			case html : printf("<td>%u</td>", score[i]); break;
		}
	switch(format)
	{
		case txt : putchar('\n'); break;
		case html: puts("\n\t\t</tr>"); break;
	}
}

void rap_print_ending(int format)
{
	switch(format)
	{
		case txt : puts("Plik wygenerowany przez SCmanager"); break;
		case html : puts("</table><br>\n<small>Plik wygenerowany \
			przez SCmanager</small>\n</body>\n</html>");
	}
}

int gen_workstation_score(const char* sdate, const char* tfrom,
	const char* tto, int ws, int* score)
{
	SCDB_RES *res_input;
	SCDB_ROW row_input;
	struct SCs_data data;

	SCdb_reset_connection(cfg.db_cfg);
	
	if( SCdb_get_input( sdate, tfrom, tto, ws ) != 0 )
		return exit_err_db;

	res_input = SCdb_store_result();
	
	SCs_analis_reset();

	while ((row_input = SCdb_fetch_row(res_input)) != NULL)
	{
		data.input = atoi(row_input[0]);
		atotms2(row_input[1], &(data.time));
		data.code = ws;
		score[SCs_data_analis(&data)] += 1;
	}

	return 0;
}

int gen_workers_day_score(const char* sdate, const char* first_name,
	const char* last_name, int* score)
{
	SCDB_RES *res_work;
	SCDB_ROW row_work;
	char tfrom[TIMELEN], tto[TIMELEN];
	int res;
	
	SCdb_reset_connection(cfg.db_cfg);

	if( SCdb_get_work( first_name, last_name, sdate ) != 0 )
		return exit_err_db;

	res_work = SCdb_store_result();

	while ((row_work = SCdb_fetch_row(res_work)) != NULL)
	{
		switch(atoi(row_work[1]))
		{
			case 1 :
			{
				strcpy(tfrom, cfg.fstart);
				strcpy(tto, cfg.fend);
			} break;
			case 2 :
			{
				strcpy(tfrom, cfg.sstart);
				strcpy(tto, cfg.send);
			} break;
			case 3 :
			{
				strcpy(tfrom, cfg.tstart);
				strcpy(tto, cfg.tend);
			} break;
			default :
			{
				fprintf(stderr, "Nieznana zmiana #%u\n",
					atoi(row_work[1]));
				SCdb_free_result(res_work);
				return exit_err_data;
			}
		}

		res = gen_workstation_score(sdate, tfrom, tto,
			atoi(row_work[0]), score);
		if(res != 0)
		{
			SCdb_free_result(res_work);
			return res;
		}
	}

	SCdb_free_result(res_work);
	return 0;
}

int generate_day_raport(const char* sdate, const char* strformatat)
{
	SCDB_RES *res_worker;
	SCDB_ROW row_worker;
	char first_name[40], last_name[40];
	int 	*score,
		format, i, res;
	
	if(strcmp(strformatat, SCM_FORMAT_TXT) == 0)
		format = txt;
	else if(strcmp(strformatat, SCM_FORMAT_HTML) == 0)
		format = html;
	else
	{
		fputs("Nieznany format\n", stderr);
		return 1 /*TODO*/;
	}

	rap_print_header(format, "Pracownicy - pojedynczy dzien", sdate);
	rap_print_first_row(format);

	if( SCdb_get_workers() != 0 )
		return exit_err_db;
	
	res_worker = SCdb_store_result();

	score = malloc(sizeof(int)*SC_score_numb);
	
	while ((row_worker = SCdb_fetch_row(res_worker)) != NULL)
	{
		strcpy(first_name, row_worker[0]);
		strcpy(last_name, row_worker[1]);

		rap_print_first_col(format, first_name, last_name);

		for(i = 0; i < SC_score_numb; i += 1)
			score[i] = 0;
		
		res = gen_workers_day_score(sdate, first_name, last_name,
			score);
		if(res != 0)
		{
			SCdb_free_result(res_worker);
			return res;
		}

		rap_print_score_col(format, score);
	}
	SCdb_free_result(res_worker);

	rap_print_ending(format);
	
	return 0;
}

int generate_month_raport(int year, int month, const char* strformatat)
{
	SCDB_RES *res_worker;
	SCDB_ROW row_worker;
	char	strdate[DATELEN],
		first_name[40], last_name[40];
	int 	*score,
		format, i, day, res;

	if(strcmp(strformatat, SCM_FORMAT_TXT) == 0)
		format = txt;
	else if(strcmp(strformatat, SCM_FORMAT_HTML) == 0)
		format = html;
	else
	{
		fputs("Nieznany format\n", stderr);
		return 1 /*TODO*/;
	}

	sprintf(strdate, "%i-%i", year, month);
	rap_print_header(format, "Pracownicy - pojedynczy miesiac", strdate);
	rap_print_first_row(format);
	
	if ( SCdb_get_workers() != 0)
		return exit_err_db;

	res_worker = SCdb_store_result();

	score = malloc(sizeof(int)*SC_score_numb);
	
	while ((row_worker = SCdb_fetch_row(res_worker)) != NULL)
	{
		strcpy(first_name, row_worker[0]);
		strcpy(last_name, row_worker[1]);

		rap_print_first_col(format, first_name, last_name);

		for(i = 0; i < SC_score_numb; i += 1)
			score[i] = 0;
		
		for(day = 1; day <= days_in_month(year, month); day += 1)
		{
			sprintf(strdate, "%u-%u-%u", year, month, day);
			
			res = gen_workers_day_score(strdate, first_name,
				last_name, score);
			if(res != 0)
			{
				SCdb_free_result(res_worker);
				return res;
			}
		}
		
		rap_print_score_col(format, score);
	}
	SCdb_free_result(res_worker);

	rap_print_ending(format);
	
	return 0;
}

int generate_short_raport(const char* sdate, const char* from_strtime,
	const char* to_strtime, int ws, const char* strformat)
{
	int 	*score,
		format, res, i;
	char sbuf[10];

	if(strcmp(strformat, SCM_FORMAT_TXT) == 0)
		format = txt;
	else if(strcmp(strformat, SCM_FORMAT_HTML) == 0)
		format = html;
	else
	{
		fputs("Nieznany format\n", stderr);
		return 1 /*TODO*/;
	}

	rap_print_header(format, "Stanowisko - krotki przedzial czasu", sdate);
	rap_print_first_row(format);

	score = malloc(sizeof(int)*SC_score_numb);
	for(i = 0; i < SC_score_numb; i += 1)
		score[i] = 0;
	
	res = gen_workstation_score(sdate, from_strtime, to_strtime, ws,
		score);
	if(res != 0)
		return res;
	
	sprintf(sbuf, "%i", ws);
	rap_print_first_col(format, "Stanowisko nr", sbuf);
	rap_print_score_col(format, score);

	rap_print_ending(format);
	
	return 0;
}
