/*
 * scwl2db - transfers data from the transfer directory do
 * the SCdb database
 */

#include <stdlib.h>
#include <stdio.h>

#include <mysql/mysql.h>

#include "scdata.h"
#include "manage/scdb.h"
#include "manage/scmanage_common.h"

#include "wlcstdc.h"

#include "iniparser.h"

struct SCwl2db_cfg
{
	dictionary *d;
	char* trans_dir;
	struct SCdb_cfg *db_cfg;
} cfg;

int init_cfg_and_SCdb();
void free_cfg_and_SCdb();

int make_trans(const char* trans_dir, int erase, int over);

int main( int argc, char **argv)
{
	int res;

	atexit(free_cfg_and_SCdb);

	res = init_cfg_and_SCdb();
	
	switch(res)
	{
		case exit_err_ini : fputs("Plik konfiguracyjny nie zawiera "
			"niezbędnych danych\n", stderr);
			break;
		case exit_err_db : fputs("Błąd przy inicjalizacji dazy danych",
			stderr);
			break;
		case 0 :
		{
			if(argc == 3)
			{
				res = make_trans(cfg.trans_dir,
					atoi(argv[1]), atoi(argv[2]));
			}
			else if(argc == 4)
			{
				res = make_trans(argv[3],
					atoi(argv[1]), atoi(argv[2]));
			}
			else
			{
				fputs("Zla ilosc argumentów\n", stderr);
				res = exit_err_argc;
			}
		}	break;
		default : fputs( "Nieznany błąd", stderr );
	}
	
	/*free_cfg_and_SCdb(); is in atexit()*/
	return res;
} 

int init_cfg_and_SCdb()
{
	cfg.d = iniparser_load("scwl2db.ini");
	if(!cfg.d)
		return exit_err_ini;
	
	cfg.trans_dir = iniparser_getstr(cfg.d, "main:trans_dir");
	if(!cfg.trans_dir)
		return exit_err_ini;

	cfg.db_cfg = make_SCdb_cfg("scdb.ini");
	if( !cfg.db_cfg )
		return exit_err_ini;

	if( SCdb_init(cfg.db_cfg) != 0 )
		return exit_err_db;

	return 0;
}

void free_cfg_and_SCdb()
{
	iniparser_freedict(cfg.d);
	SCdb_free();
	fflush(stdout);
}

int make_trans(const char* trans_dir, int erase, int over)
{
	int	res = 0,
		query_err = 0;
	const int max_query_err = 3;
	char 	strb[100],
		worklog_dir[100],
		wl_date[DATELEN];
	FILE	*terror, *wlset, *worklog, *fmd5, *errorlog;
	unsigned char	md5sumA[MD5LEN],
			md5sumB[MD5LEN];
	struct SCs_data data;

	strcpy(strb, trans_dir);
	strcatch(strb, dir_sep);
	terror = fopen(strcat(strb, "terror"), "rb");
	if(terror)
	{
		fclose(terror);
		fputs("Wystąpił krytyczny błąd podczas transferu \
			po stronie grupy produkcyjnej\n", stderr);
		return exit_err_data;
	}

	strcpy(strb, trans_dir);
	strcatch(strb, dir_sep);
	wlset = fopen(strcat(strb, SC_wlset_name), "r");
	if(!wlset)
	{
		fprintf(stderr, "%c Brak pliku %s\n", dir_sep, strb);
		return exit_err_data;
	}

	fgets(wl_date, DATELEN, wlset);
	while(!feof(wlset))
	{
		if(fgetc(wlset) != '\n')
		{
			fputs("Plik wl.set jest nie zgodny z formatem\n",
				stderr);
			return exit_err_data;
		}

		if( SCdb_insert_wl(wl_date) != 0 )
		{
			fputs("Nie mozna zapisac danych do bazy danych", stderr);
			return exit_err_data;
		}
		
		sprintf(strb, "%s%cworklogs/%s.wl", trans_dir, dir_sep, wl_date);
		strcpy(worklog_dir, strb);
		worklog = fopen(worklog_dir, "rb");
		if(!worklog)
		{
			fprintf(stderr, "Brak pliku %s\n", worklog_dir);
			res = exit_err_data;
			continue;
		}

		fmd5 = fopen(strcat(strb, ".md5"), "rb");
		if(!fmd5)
		{
			fprintf(stderr, "Brak pliku %s\n", strb);
			res = exit_err_data;
			fclose(worklog);
			continue;
		}

		md5file(worklog, md5sumA);
		fread(md5sumB, sizeof(unsigned char)*MD5LEN, 1, fmd5);
		fclose(fmd5);
		if( memcmp((void*)md5sumA, (void*)md5sumB,
			MD5LEN*sizeof(unsigned char)) != 0 )
		{
			fprintf(stderr, "Plik %s jest zepsuty\n", worklog_dir);
			res = exit_err_data;
			fclose(worklog);
			continue;
		}

		fclose(worklog);
		worklog = fopen(worklog_dir, "rb");
		fputs(wl_date, stdout);
		fputc('\n', stdout);
		while(!feof(worklog))
		{
			if( SCs_data_fread(worklog, &data) != 0)
				continue;
			
			if( SCdb_insert_SCs_data(over, wl_date, &data) != 0)
			{
				fprintf(stderr, "Nie udana próba zapisu rekordu"
						" danych z %s\n", worklog_dir);
				res = exit_err_data;
				query_err += 1;
				if(query_err > max_query_err)
				{
					fprintf( stderr,
						"Przekroczono maksymalną ilość "
						"nieudanych zapytań SQL\n");
					res = exit_err_data;
					continue;
				}
			}
		}
		
		fgets(wl_date, DATELEN, wlset);
	}

	sprintf(strb, "%s%c%s", trans_dir, dir_sep, SC_errorlog_name);
	errorlog = fopen(strb, "rb");
	if(errorlog)
	{
		struct SCr_errno SCerr;
		struct tm* tm_time;
		
		puts(SCM_SEPARATOR);
		SCr_err_fread(errorlog, &SCerr);
		while(!feof(errorlog))
		{
			tm_time = localtime(&(SCerr.time));
			printf( "%u.%u.%u %u:%u:%u %s %u\n",
				tm_time->tm_year + 1900, tm_time->tm_mon,
				tm_time->tm_mday, tm_time->tm_hour,
				tm_time->tm_min, tm_time->tm_sec,
				SCr_err_to_str(SCerr.type), SCerr.code );
			SCr_err_fread(errorlog, &SCerr);
		}
		fclose(errorlog);
	}

	fclose(wlset);
	if(erase)
	{
		sprintf(strb, "rmdir %s", trans_dir);
		system(strb);
	}
	return res;
}
