/** @file scdata_l.h
 * Functions and structures used to:
 * - log data records form SCslaves to worklog files,
 * - log SCrunner process activity to proclog files.
 */

#ifndef SCDATA_L_H_
#define SCDATA_L_H_

#include <stdio.h>
#include <string.h>

#include "scdata.h"
#include "work/scrunner_msg.h"
#include "sccommon.h"

#include "wlcstdc.h"

extern int	SCr_not_loged,	/**<
				 * 0 if last SCslave data record
				 * was logged successful; if not
				 * then the number unsuccessful
				 * data logs
				 */
		SCr_max_not_loged;	/**<
				 * Maximum number of unsuccessful
				 * data logs after which a critical
				 * error is generated
				 */

/** Proclogs directory path */
extern const char* SCr_proclog_dir;

/** 1 if proclog files should be generated; 0 if not */
extern int SCr_proc_log_on;

/** Internal process type */
enum SCr_proc_type
{
	SCr_proc_data,		/**< Login SCslave data record */
	SCr_proc_error,		/**< Error rised */
	SCr_proc_user		/**< User interaction */
};

/** User interaction options */
enum SCr_user_opt
{
	SCr_user_trans_start,	/**< Begin transfer */
	SCr_user_trans_end,	/**< End transfer */
	SCr_user_sys_off,	/**< Shutdown system */
	SCr_user_sys_on		/**< System turned on */
};

/** State of the current worklog file */
enum SCr_wl_state
{
	SCr_wl_state_err = -1,	/**< error while checking state */
	SCr_wl_state_new = 0,	/**< New file */
	SCr_wl_state_old = 1	/**< Old file */
};

/**
 * Checks the state of the current worklog file.
 * @param strdate current date in ISO format
 * @return the state from SCr_wl_state
 */
int SCr_new_wl(const char* strdate);

/**
 * Converts the user interaction code to its name.
 * @param type user interaction code from SCr_usr_opt
 * @return a pointer to a string with the name of the
 * users interaction
 */
char* SCr_user_to_str(int type);

/**
 * Logs an internal SCrunner process
 * @param proc process type from SCr_proc_type
 * @param codeA first process code
 * @param codeB second process code
 * @param strdate current date in ISO format
 */
void SCr_log_proc(int proc, int codeA, int codeB, const char* strdate);

/**
 * Logs an error.
 * @param type error type
 * @param code error code
 * @param strdate current date in ISO format
 */
void SCr_log_error(int type, int code, const char* strdate);

/**
 * Logs a data record from SCslave.
 * @param unloged data record
 * @param strdate date in ISO format
 */
int SCr_log_data(const struct SCs_data* unloged, const char* strdate);

#endif /* SCDATA_L_H_ */

