/*
 * scrunner_tui is a NCurses base front end for scrunner
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <pthread.h>
#include <unistd.h>
#include <ncurses.h>
#include <signal.h>

#include "scdata.h"
#include "analis/scdata_a.h"
#include "work/scrunner_msg.h"
#include "wlcstdc.h"

#define PCOLOR 1
#define NCOLOR 2
#define ASCOLOR 3
#define ALCOLOR 4
#define SCOLOR 5

#define KEY_ESC 033

const int s_view = 10;
const int l_view = 7;

int running = 1;

FILE* scr_pipe;
int nl_pipe = 0;
char line_pipe[SCR_MAX_LINE];

char scr_start_path[L_tmpnam];
int end_swin;

void term_hand(int s);
void free_res();
int load_background(const char* bc_file_path, const char* title);
char* save_start(char* start_path, FILE* scr_pipe);
char* load_start(char* start_text, const char* start_path);
const struct SCs_data* get_data(const char* proc_line);
void refresh_scores(/* all const */int** tab, int m, int s,
		int from, int code, int score);
void add_log(const char* proc_line, char** log, int lh_size);
void refresh_log(/* all const */char** log, int from);
void *read_pipe(void *arg);
void swin(const char* scr_start_path);
char* get_path(char* path);
void save_scores(/* all const */int** score_tab,
		int m, int s, const char* path); 

int main(int argc, char** argv)
{
	int mpid;
	char	line[SCR_MAX_LINE],
		title[SCR_MAX_LINE],
		vr[10];
	int cbuf;
	int i, j;
	const char* bc_file_path = "scrtui_inter.txt";
	const struct SCs_data* data;
	int** score_tab;
	const int lh_size = 20;
	char** log_history = malloc(sizeof(char*)*lh_size);
	int	from_s = 0,
		from_l = 0,
		code = -1,
		score = -1;
	char 	tmp_dump_path[L_tmpnam],
		usr_dump_path[SCR_MAX_LINE];
	pthread_t pipe_thread;
	
	if(signal(SIGTERM, term_hand) == SIG_ERR)
		return 1;
	if(argc == 2)
	{
		scr_pipe = fopen(argv[1], "r");
		if(!scr_pipe)
			return 2;
	}
	else
		return 1;
	
	
	for(i = 0; i < lh_size; i += 1)
	{
		log_history[i] = malloc(sizeof(char)*SCR_MAX_LINE);
		log_history[i][0] = '\0';
	}

	fgets(title, SCR_MAX_LINE, scr_pipe);
	sscanf(title, SCR_INIT_MSG, vr, &mpid);

	save_start(scr_start_path, scr_pipe);
	
	initscr();
	cbreak();
	noecho();
	nonl();
	nodelay(stdscr, TRUE);
	start_color();
	init_pair(PCOLOR, COLOR_GREEN, COLOR_BLACK);
	init_pair(NCOLOR, COLOR_YELLOW, COLOR_BLACK);
	init_pair(ASCOLOR, COLOR_RED, COLOR_WHITE);
	init_pair(ALCOLOR, COLOR_BLUE, COLOR_BLACK);
	init_pair(SCOLOR, COLOR_WHITE, COLOR_BLACK);
	clear();
	if(load_background(bc_file_path, title) == -1)
		return 2;

	score_tab = malloc(sizeof(int)*SCS_NUMB_MAX);
	for(i = 0; i < SCS_NUMB_MAX; i += 1)
	{
		score_tab[i] = malloc(sizeof(int)*SC_score_numb);
		for(j = 0; j < SC_score_numb; j += 1)
			score_tab[i][j] = 0;
	}
	
	
	refresh_scores(score_tab, SCS_NUMB_MAX, SC_score_numb, from_s, -1, -1);
	refresh();
	line[0] = '\0';
	pthread_create(&pipe_thread, NULL, read_pipe, NULL);
	while(running && (strcmp(line, SCR_END_MSG) != 0))
	{
		cbuf = getch();
		switch(cbuf)
		{
			case 's' :
			{
				if(!(SCS_NUMB_MAX < (from_s + s_view + 1)))
				{
					from_s += 1;
					refresh_scores(score_tab, SCS_NUMB_MAX, SC_score_numb, from_s, code, score);
				}
				else
					beep();
			}	break;
			case 'w' :
			{
				if(from_s > 0)
				{
					from_s -= 1;
					refresh_scores(score_tab, SCS_NUMB_MAX, SC_score_numb, from_s, code, score);
				}
				else
					beep();
			}	break;
			case 'S' :
			{
				if(!(lh_size < (from_l + l_view + 1)))
				{
					from_l += 1;
					refresh_log(log_history, from_l);
				}
				else
					beep();
			}	break;
			case 'W' :
			{
				if(from_l > 0)
				{
					from_l -= 1;
					refresh_log(log_history, from_l);
				}
				else
					beep();
			}	break;
			case 'z' :
			{
				char cmd[40];
				sprintf(cmd, "kill -s USR1 %u", mpid);
				system(cmd);
			}	break;
			case 'x' :
			{
				char cmd[40];
				sprintf(cmd, "kill -s TERM %u", mpid);
				system(cmd);
				running = 0;
			}	break;
			case 'X' :
			{
				char cmd[40];
				sprintf(cmd, "kill -s KILL %u", mpid);
				system(cmd);
				running = 0;
			}	break;
			case 'e' :
			{
				tmpnam(tmp_dump_path);
				refresh();
				scr_dump(tmp_dump_path);
				swin(scr_start_path);
				refresh();
				clear();
				refresh();
				scr_restore(tmp_dump_path);
			}	break;	
			case 'q' :
			{
				char* ok;
				tmpnam(tmp_dump_path);
				refresh();
				scr_dump(tmp_dump_path);
				ok = get_path(usr_dump_path);
				clear();
				refresh();
				scr_restore(tmp_dump_path);
				if(ok)
					save_scores(score_tab, SCS_NUMB_MAX, SC_score_numb, usr_dump_path);
			}	break;
		}
				
		if(nl_pipe && running)
		{
			pthread_join(pipe_thread, NULL);
			data = get_data(line_pipe);
			if(data)
			{
				code = data->code-1;
				score = SCs_data_analis(data);
				score_tab[code][score] += 1;
				refresh_scores(score_tab, SCS_NUMB_MAX, SC_score_numb, from_s, code, score);
			}
	
			add_log(line_pipe, log_history, lh_size);
			refresh_log(log_history, from_l);
			nl_pipe = 0;
			pthread_create(&pipe_thread, NULL, read_pipe, NULL);
		}
		refresh();
	}
	
	endwin();
	fclose(scr_pipe);
	free(score_tab);
	free(log_history);
	return 0;
}

void term_hand(int s)
{
	running = 0;
}

int load_background(const char* bc_file_path, const char* title)
{
	FILE* bc_file = fopen(bc_file_path, "r");
	char line[SCR_MAX_LINE];
	int count = 0;

	attron(COLOR_PAIR(SCOLOR));
	
	if(!bc_file)
		return -1;
	
	fgets(line, SCR_MAX_LINE, bc_file);
	while(!feof(bc_file))
	{
		
		mvprintw(count, 1, "%s", line);
		count += 1;
		fgets(line, SCR_MAX_LINE, bc_file);
	}

	mvprintw(1, 3, "%s", title);
	fclose(bc_file);
	return count;
}

char* save_start(char* start_path, FILE* scr_pipe)
{
	FILE* scr_start;
	char line[SCR_MAX_LINE];
	tmpnam(start_path);
	scr_start = fopen(start_path, "w");
	
	fgets(line, SCR_MAX_LINE, scr_pipe);
	while(strcmp(line, SCR_PROCLOG_START_MSG) != 0)
	{
		if(scr_start)
			fputs(line, scr_start);
		
		if(strcmp(line, SCR_END_MSG) == 0)
			return NULL;
		fgets(line, SCR_MAX_LINE, scr_pipe);
	}
	
	if(scr_start)
		fclose(scr_start);
	return start_path;
}

char* load_start_line(char* start_line, const char* start_path)
{
	static FILE* sf = NULL;
	
	if(!sf)
		sf = fopen(start_path, "r");
	
	if(feof(sf))
	{
		fclose(sf);
		sf = NULL;
		return NULL;
	}

	fgets(start_line, SCR_MAX_LINE, sf);
	
	return start_line;
}

const struct SCs_data* get_data(const char* proc_line)
{
	static struct SCs_data data;
	char proc_type[10], strinput[20];
	if(sscanf(proc_line, "%u:%u:%u %s\tcode:%u\tinput:%s",
		&(data.time.hour), &(data.time.min), &(data.time.sec),
		proc_type, &(data.code), strinput) != 6)
		return NULL;
	if(strcmp(proc_type, "DATA:") != 0)
		return NULL;
	data.input = abtoi(strinput, strlen(strinput));
	return &data;
}

void refresh_scores(/* all const */int** tab, int m, int s,
				int from, int code, int score)
{
	int	i, j;

	for(i = 0; i < s_view; i += 1)
	{
		if((i+from)%2)
			attron(COLOR_PAIR(NCOLOR));
		else
			attron(COLOR_PAIR(PCOLOR));
		mvprintw(i+5, 8, "%2u", i+1+from);
		for(j = 0; j < s; j += 1)
		{
			if(((i+from) == code) && (j == score))
				attron(COLOR_PAIR(ASCOLOR));
			mvprintw(i+5, 13 + j*7, "%4u", tab[i+from][j]);
			if((i+from)%2)
				attron(COLOR_PAIR(NCOLOR));
			else
				attron(COLOR_PAIR(PCOLOR));
		}
	}
}

void add_log(const char* proc_line, char** log, int lh_size)
{
	int i;

	for(i = lh_size - 1; i >= 1; i -= 1)
		strcpy(log[i], log[i - 1]);

	strcpy(log[0], proc_line);
}

void refresh_log(char** log, int from)
{
	int i;
	for(i = 0; i < l_view; i += 1)
	{
		if((i+from)%2)
			attron(COLOR_PAIR(NCOLOR));
		else
			attron(COLOR_PAIR(PCOLOR));
		
		mvprintw(i+16, 3, "%-.50s", log[i+from]);
	}
}

void *read_pipe(void *argv)
{
	while(nl_pipe)
		usleep(250);
	fgets(line_pipe, SCR_MAX_LINE, scr_pipe);
	nl_pipe = 1;
	return NULL;
}

void swin(const char* scr_start_path)
{
	char line[SCR_MAX_LINE];
	int i = 0;

	attron(COLOR_PAIR(SCOLOR));
	mvprintw(6, 10, "+--(SCrunner initialization)------------------------+");
	while(load_start_line(line, scr_start_path) != NULL)
	{
		i += 1;
		mvprintw(6+i, 10, "|                                                   |");
		mvprintw(6+i, 12, "%-.30s", line);
	}
	mvprintw(6+i, 10, "+---------------------------------------------------+");
	while(getch() != 'e')
		usleep(300);
}

char* get_path(char* path)
{
	int c, i = 0;
	
	attron(COLOR_PAIR(SCOLOR));
	mvprintw(10, 10, "+--(Save scores)------------------------------+");
	mvprintw(11, 10, "| Path:                                       |");
	mvprintw(12, 10, "+---------------------------------------------+");
	move(11, 21);
	refresh();
	echo();
	nl();
	do
	{
		c = getch();
		usleep(100);
		switch(c)
		{
			case KEY_ESC :
			{
				nonl();
				noecho();
				return NULL;
			}	break;
			case ERR : {} break;
			default :
			{
				path[i] = (char)c;
				i += 1;
			}
		}
	}  while(c != '\n');
	nonl();
	path[i-1] = '\0';
	noecho();
	
	return path;
}

void save_scores(/* all const */int** score_tab, 
		int m, int s, const char* path)
{
	FILE* sf = fopen(path, "w");
	int i, j;

	if(!sf)
		return;
	
	fputc('\t', sf);
	for(j = 0; j < s; j += 1)
		fprintf(sf, "%5u", j);
	fputc('\n', sf);
	for(i = 0; i < m; i += 1)
	{
		fprintf(sf, "%2u\t", i+1);
		for(j = 0; j < s; j += 1)
			fprintf(sf, "%5u", score_tab[i][j]);
		fputc('\n', sf);
	}
	fclose(sf);
}
	
